# -*- coding: utf-8 -*-
# Módulo pydatajson
# Conjunto de herramientas para validar y manipular la información presente en
# el archivo `data.json` de un Portal de Datos
from pydatajson import DataJson

__author__ = """Datos Argentina"""
__email__ = 'datos@modernizacion.gob.ar'
__version__ = '0.1.0'
